/**
* @license Apache-2.0
*
* Copyright (c) 2024 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

// MODULES //

var dtype = require( '@stdlib/array/dtype' );
var dtype2wasm = require( './../../../base/dtype2wasm' );


// MAIN //

/**
* Returns the WebAssembly data type for a provided array.
*
* @param {Collection} array - input array
* @returns {string} data type
*
* @example
* var Float64Array = require( '@stdlib/array/float64' );
*
* var dt = array2dtype( new Float64Array( 4 ) );
* // returns 'float64'
*
* @example
* var dt = array2dtype( [ 0, 1, 0, 1 ] );
* // returns 'float64'
*
* @example
* var Float32Array = require( '@stdlib/array/float32' );
*
* var dt = array2dtype( new Float32Array( 4 ) );
* // returns 'float32'
*/
function array2dtype( array ) {
	return dtype2wasm( dtype( array ) || 'generic' );
}


// EXPORTS //

module.exports = array2dtype;
