/**
* @license Apache-2.0
*
* Copyright (c) 2024 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

/*
* When adding modules to the namespace, ensure that they are added in alphabetical order according to module name.
*/

// MODULES //

var setReadOnly = require( '@stdlib/utils/define-read-only-property' );


// MAIN //

/**
* Top-level namespace.
*
* @namespace ns
*/
var ns = {};

/**
* @name array2dtype
* @memberof ns
* @readonly
* @type {Function}
* @see {@link module:@stdlib/wasm/base/array2dtype}
*/
setReadOnly( ns, 'array2dtype', require( './../../base/array2dtype' ) );

/**
* @name arrays2ptrs
* @memberof ns
* @readonly
* @type {Function}
* @see {@link module:@stdlib/wasm/base/arrays2ptrs}
*/
setReadOnly( ns, 'arrays2ptrs', require( './../../base/arrays2ptrs' ) );

/**
* @name dtype2wasm
* @memberof ns
* @readonly
* @type {Function}
* @see {@link module:@stdlib/wasm/base/dtype2wasm}
*/
setReadOnly( ns, 'dtype2wasm', require( './../../base/dtype2wasm' ) );

/**
* @name strided2object
* @memberof ns
* @readonly
* @type {Function}
* @see {@link module:@stdlib/wasm/base/strided2object}
*/
setReadOnly( ns, 'strided2object', require( './../../base/strided2object' ) );


// EXPORTS //

module.exports = ns;
